from abc import ABC, abstractmethod
from typing import Any


class MediaRepository(ABC):
    @abstractmethod
    def retrieve(self, media_id: str, width: int = None) -> Any:
        raise NotImplementedError



from abc import ABC, abstractmethod


class AnnotationTaskProvider(ABC):
    @abstractmethod
    def get_number_of_annotations(self):
        raise NotImplementedError

    @abstractmethod
    def get_number_pending_annotations(self):
        raise NotImplementedError

    @abstractmethod
    def get_total_annotations(self):
        raise NotImplementedError

    @abstractmethod
    def get_status(self):
        raise NotImplementedError

    @abstractmethod
    def next(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def previous(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def confirm_current_task(self, task_id: str):
        raise NotImplementedError



import json
from uuid import uuid4

import numpy as np
from datetime import datetime
from typing import List, Dict


class AnnotationSession:
    @staticmethod
    def create(ids: List[str], users: List[str], number_of_annotations_per_media: int):
        created_at = datetime.utcnow().strftime("%B %d %Y - %H:%M:%S")
        users = users
        number_of_ids = len(ids)
        number_of_users = len(users)
        number_of_annotations_per_media = number_of_annotations_per_media

        ids_repeated = np.repeat(np.array(ids), number_of_annotations_per_media)
        tasks_list = [[] for i in range(number_of_users)]
        for i, media_id in enumerate(ids_repeated):
            row_id = i % number_of_users
            tasks_list[row_id].append(media_id)

        tasks = {}
        for i, user in enumerate(users):
            tasks[user] = tasks_list[i]

        return AnnotationSession(
            str(uuid4()),
            created_at,
            number_of_ids,
            number_of_users,
            number_of_annotations_per_media,
            tasks,
        )

    def __init__(
        self,
        id: str,
        created_at: str,
        number_of_ids: int,
        number_of_users: int,
        number_of_annotations_per_media: int,
        tasks: Dict[str, List[str]],
    ):
        self.id = id
        self.created_at = created_at
        self.number_of_ids = number_of_ids
        self.number_of_users = number_of_users
        self.number_of_annotations_per_media = number_of_annotations_per_media
        self.tasks = tasks

    def to_dict(self):
        return {
            "id": self.id,
            "created_at": self.created_at,
            "number_of_ids": self.number_of_ids,
            "number_of_users": self.number_of_users,
            "number_of_annotations_per_media": self.number_of_annotations_per_media,
            "tasks": self.tasks,
        }

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=4)

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()

    def save(self, filename):
        with open(filename, "w") as outfile:
            json.dump(self.to_dict(), outfile, indent=4)

    @staticmethod
    def from_filename(filename):
        with open(filename, "r") as infile:
            kdict = json.load(infile)
            return AnnotationSession(
                kdict.get("id"),
                kdict.get("created_at"),
                kdict.get("number_of_ids"),
                kdict.get("number_of_users"),
                kdict.get("number_of_annotations_per_media"),
                kdict.get("tasks"),
            )

    def get_tasks_by_user(self, username: str) -> List[str]:
        return self.tasks.get(username)



class AnnotationTaskStatus:
    def __init__(
        self,
        number_of_annotations: int,
        total_annotations: int,
        current_task_id: str = None,
    ):
        self.number_of_annotations = number_of_annotations
        self.total_annotations = total_annotations
        self.current_task_id = current_task_id

    def get_progress(self):
        return f"{self.number_of_annotations+1} of {self.total_annotations}"



from ipywidgets import Output

from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository
from chosco.media.retriever.domain.media_retriever import MediaRetriever
from chosco.view.interactor.domain.view_interactor import ViewInteractor


class AnnotatorBuilder:
    def __init__(
        self,
        username: str,
        retriever: MediaRetriever,
        repository: AnnotationRepository,
        view_interactor: ViewInteractor,
    ):
        self.username = username
        self.retriever = retriever
        self.repository = repository
        self.view_interactor = view_interactor
        self.current_media_id = None

        def update_item(media_id, image):
            self.current_media_id = media_id
            status = self.retriever.get_status()
            self.view_interactor.update_header_box(media_id, image, status)
            try:
                annotation = self.repository.retrieve(media_id, self.username)
                self.view_interactor.update_annotation_box(annotation.metadata)
            except ValueError:
                self.view_interactor.clear_annotation_box()

        def previous_item():
            media_id, image = self.retriever.previous_media()
            if media_id:
                update_item(media_id, image)

        def next_item():
            media_id, image = self.retriever.next_media()
            if media_id:
                update_item(media_id, image)

        def confirm_annotation(create_metadata):
            metadata = create_metadata()
            annotation = Annotation(self.current_media_id, self.username, metadata)
            self.repository.save(annotation)
            self.repository.show()
            self.retriever.confirm(self.current_media_id)

        self.view_interactor.set_callbacks(
            {
                "next_item": next_item,
                "previous_item": previous_item,
                "confirm_annotation": confirm_annotation,
            }
        )

    def build(self, output=None):
        if not output:
            output = Output()
        self.view_interactor.execute(output)
        return output



from abc import ABC, abstractmethod

from chosco.annotation.domain.annotation import Annotation


class AnnotationRepository(ABC):
    @abstractmethod
    def save(self, annotation: Annotation):
        raise NotImplementedError

    @abstractmethod
    def retrieve(self, annotation_id: str, username: str) -> Annotation:
        raise NotImplementedError

    @abstractmethod
    def show(self):
        raise NotImplementedError



import json
import os

from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository


class JsonFolderAnnotationRepository(AnnotationRepository):
    def __init__(self, output_folder: str):
        self.output_folder = output_folder
        os.makedirs(self.output_folder, exist_ok=True)
        self.username_filenames = {}
        self.user_annotations = {}

    def save(self, annotation: Annotation):
        if annotation.username not in self.username_filenames:
            filename = f"{self.output_folder}/{annotation.username}.json"
            self.username_filenames[annotation.username] = filename

        filename = self.username_filenames.get(annotation.username)
        if os.path.isfile(filename):
            with open(filename, "r") as infile:
                self.user_annotations = json.load(infile)

        self.user_annotations[annotation.id] = annotation.metadata
        with open(filename, "w") as outfile:
            json.dump(self.user_annotations, outfile, indent=4)

    def retrieve(self, annotation_id: str, username: str) -> Annotation:
        if username not in self.username_filenames:
            raise ValueError(
                f"username {username} not exist in [{self.username_filenames.keys()}]"
            )

        filename = self.username_filenames.get(username)
        if not os.path.isfile(filename):
            raise ValueError(f"username filename {filename} not exist")

        with open(filename, "r") as infile:
            user_annotations = json.load(infile)
            if annotation_id not in user_annotations:
                raise ValueError(
                    f"annotation_id {annotation_id} not exist in [{user_annotations}]"
                )

            return Annotation(
                id=annotation_id,
                username=username,
                metadata=user_annotations.get(annotation_id),
            )

    def show(self):
        print(json.dumps(self.username_filenames, indent=4))
